

from just_semantic_search.splitters.abstract_splitters import AbstractTextSplitter


class RemoteTextSplitter(AbstractTextSplitter):
    """
    Splitter that uses a remote text splitter to split text.
    """
    pass

